
' Netzfreq-2.bas
' Fuses: L = 0xff    H = 0xdf    E = 0xfe
' Compiler BASCOM 2.0.7.5 DEMO 

' Das Programm ermittelt die genaue Netzfrequenz und gibt einen Zählwert
' per serieller Schnittstelle aus.
'
' Eine kleine Antenne an PB.2 nimmt das 50 Hz-Signal auf.
' Timer0 zählt 50 Flankenwechsel Hi->Lo des Signals (etwa 1 Sekunde) und
' wirft dann einen Interrupt.

' Timer1 zählt den 16 MHz-Quarztakt / 128 = 125 kHz. Nach 255 Schritten 
' läuft er über, die Überläufe (488 / sec) lösen jeweils einen Interrupt 
' aus und werden in der Variablen "Ovfcnt" addiert.

' Nach dem Timer0-Interrupt werden der Zählerstand von Timer1 und die 
' Überläufe zusammengesetzt (Registerinhalt TCNT1 plus 256 * Ovfcnt) und
' über die serielle Schnittstelle ausgegeben.
 
' Der ausgegebene Wert beträgt etwa 125000. Die Umrechnung in die 
' Netzfrequenz erfolgt im PC-Auswerteprogramm.


$regfile = "attiny85.dat"
$crystal = 16000000
$hwstack = 40
$swstack = 4
$framesize = 4


Dim F As Long
Dim Ovfcnt As Long

Config Portb.0 = Output
Config PortB.1 = Output

' bei Verwendung eines RS232-USB-Wandlers:
Open "comb.1:115200,8,n,1,INVERTED" For Output As #1
' bei Verwendung eines TTL-USB-Wandlers stattdessen:
' Open "comb.1:115200,8,n,1" For Output As #1

Config Timer0 = Counter , Edge = Falling
Enable Timer0
On Timer0 Tim0isr

Config Timer1 = Timer , Prescale = 128
Enable Timer1
On Timer1 Tim1isr

Enable Interrupts


Do

  If Ovfcnt < 31 Then             ' erzeugt Blinksignal
    PortB.0 = 1
  Else
    PortB.0 = 0
  End If
Loop


Tim0isr:
  F = TCNT1
  TCNT1 = 0                       ' Counter1 zuruecksetzen
  TCNT0 = 206                     ' Counter0 zuruecksetzen 
  Shift Ovfcnt, Left, 8           ' ist schneller als Multiplikation mit 256
  F = F + Ovfcnt                  ' Werte um 125000
  Ovfcnt = 0
  Print #1 , F 
Return


Tim1isr:
  Incr Ovfcnt                     ' Ovfcnt: Overflow-Counter
Return

End
